@@ -1,5 +1,5 @@
 /* Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0 */
-/* For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt */
+/* For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt */
 
 /* C-based Tracer for coverage.py. */
 
@@ -13,7 +13,7 @@
 static int
 pyint_as_int(PyObject * pyint, int *pint)
 {
-    int the_int = MyInt_AsInt(pyint);
+    int the_int = (int)PyLong_AsLong(pyint);
     if (the_int == -1 && PyErr_Occurred()) {
         return RET_ERROR;
     }
@@ -39,7 +39,7 @@ CTracer_intern_strings(void)
     int ret = RET_ERROR;
 
 #define INTERN_STRING(v, s)                     \
-    v = MyText_InternFromString(s);             \
+    v = PyUnicode_InternFromString(s);          \
     if (v == NULL) {                            \
         goto error;                             \
     }
@@ -103,6 +103,7 @@ CTracer_dealloc(CTracer *self)
     Py_XDECREF(self->should_start_context);
     Py_XDECREF(self->switch_context);
     Py_XDECREF(self->context);
+    Py_XDECREF(self->disable_plugin);
 
     DataStack_dealloc(&self->stats, &self->data_stack);
     if (self->data_stacks) {
@@ -132,14 +133,15 @@ indent(int n)
 
 static BOOL logging = FALSE;
 /* Set these constants to be a file substring and line number to start logging. */
-static const char * start_file = "tests/views";
-static int start_line = 27;
+static const char * start_file = "nested.py";
+static int start_line = 1;
 
 static void
-showlog(int depth, int lineno, PyObject * filename, const char * msg)
+CTracer_showlog(CTracer * self, int lineno, PyObject * filename, const char * msg)
 {
     if (logging) {
-        printf("%s%3d ", indent(depth), depth);
+        int depth = self->pdata_stack->depth;
+        printf("%x: %s%3d ", (int)self, indent(depth), depth);
         if (lineno) {
             printf("%4d", lineno);
         }
@@ -147,8 +149,8 @@ showlog(int depth, int lineno, PyObject * filename, const char * msg)
             printf("    ");
         }
         if (filename) {
-            PyObject *ascii = MyText_AS_BYTES(filename);
-            printf(" %s", MyBytes_AS_STRING(ascii));
+            PyObject *ascii = PyUnicode_AsASCIIString(filename);
+            printf(" %s", PyBytes_AS_STRING(ascii));
             Py_DECREF(ascii);
         }
         if (msg) {
@@ -158,9 +160,9 @@ showlog(int depth, int lineno, PyObject * filename, const char * msg)
     }
 }
 
-#define SHOWLOG(a,b,c,d)    showlog(a,b,c,d)
+#define SHOWLOG(l,f,m)    CTracer_showlog(self,l,f,m)
 #else
-#define SHOWLOG(a,b,c,d)
+#define SHOWLOG(l,f,m)
 #endif /* TRACE_LOG */
 
 #if WHAT_LOG
@@ -172,22 +174,38 @@ static int
 CTracer_record_pair(CTracer *self, int l1, int l2)
 {
     int ret = RET_ERROR;
+    PyObject * packed_obj = NULL;
+    uint64 packed = 0;
 
-    PyObject * t = NULL;
-
-    t = Py_BuildValue("(ii)", l1, l2);
-    if (t == NULL) {
+    // Conceptually, data is a set of tuples (l1, l2), but that literally
+    // making a set of tuples would require us to construct a tuple just to
+    // see if we'd already recorded an arc.  On many-times-executed code,
+    // that would mean we construct a tuple, find the tuple is already in the
+    // set, then discard the tuple.  We can avoid that overhead by packing
+    // the two line numbers into one integer instead.
+    // See collector.py:flush_data for the Python code that unpacks this.
+    if (l1 < 0) {
+        packed |= (1LL << 40);
+        l1 = -l1;
+    }
+    if (l2 < 0) {
+        packed |= (1LL << 41);
+        l2 = -l2;
+    }
+    packed |= (((uint64)l2) << 20) + (uint64)l1;
+    packed_obj = PyLong_FromUnsignedLongLong(packed);
+    if (packed_obj == NULL) {
         goto error;
     }
 
-    if (PyDict_SetItem(self->pcur_entry->file_data, t, Py_None) < 0) {
+    if (PySet_Add(self->pcur_entry->file_data, packed_obj) < 0) {
         goto error;
     }
 
     ret = RET_OK;
 
 error:
-    Py_XDECREF(t);
+    Py_XDECREF(packed_obj);
 
     return ret;
 }
@@ -231,7 +249,7 @@ CTracer_set_pdata_stack(CTracer *self)
 
             /* A new concurrency object.  Make a new data stack. */
             the_index = self->data_stacks_used;
-            stack_index = MyInt_FromInt(the_index);
+            stack_index = PyLong_FromLong((long)the_index);
             if (stack_index == NULL) {
                 goto error;
             }
@@ -277,48 +295,6 @@ error:
  * Parts of the trace function.
  */
 
-static int
-CTracer_check_missing_return(CTracer *self, PyFrameObject *frame)
-{
-    int ret = RET_ERROR;
-
-    if (self->last_exc_back) {
-        if (frame == self->last_exc_back) {
-            /* Looks like someone forgot to send a return event. We'll clear
-               the exception state and do the RETURN code here.  Notice that the
-               frame we have in hand here is not the correct frame for the RETURN,
-               that frame is gone.  Our handling for RETURN doesn't need the
-               actual frame, but we do log it, so that will look a little off if
-               you're looking at the detailed log.
-
-               If someday we need to examine the frame when doing RETURN, then
-               we'll need to keep more of the missed frame's state.
-            */
-            STATS( self->stats.missed_returns++; )
-            if (CTracer_set_pdata_stack(self) < 0) {
-                goto error;
-            }
-            if (self->pdata_stack->depth >= 0) {
-                if (self->tracing_arcs && self->pcur_entry->file_data) {
-                    if (CTracer_record_pair(self, self->pcur_entry->last_line, -self->last_exc_firstlineno) < 0) {
-                        goto error;
-                    }
-                }
-                SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "missedreturn");
-                self->pdata_stack->depth--;
-                self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
-            }
-        }
-        self->last_exc_back = NULL;
-    }
-
-    ret = RET_OK;
-
-error:
-
-    return ret;
-}
-
 static int
 CTracer_handle_call(CTracer *self, PyFrameObject *frame)
 {
@@ -341,7 +317,6 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     CFileDisposition * pdisp = NULL;
 
     STATS( self->stats.calls++; )
-    self->activity = TRUE;
 
     /* Grow the stack. */
     if (CTracer_set_pdata_stack(self) < 0) {
@@ -353,7 +328,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
 
     /* See if this frame begins a new context. */
-    if (self->should_start_context && self->context == Py_None) {
+    if (self->should_start_context != Py_None && self->context == Py_None) {
         PyObject * context;
         /* We're looking for our context, ask should_start_context if this is the start. */
         STATS( self->stats.start_context_calls++; )
@@ -384,7 +359,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     }
 
     /* Check if we should trace this line. */
-    filename = frame->f_code->co_filename;
+    filename = MyFrame_GetCode(frame)->co_filename;
     disposition = PyDict_GetItem(self->should_trace_cache, filename);
     if (disposition == NULL) {
         if (PyErr_Occurred()) {
@@ -503,7 +478,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
             if (PyErr_Occurred()) {
                 goto error;
             }
-            file_data = PyDict_New();
+            file_data = PySet_New(NULL);
             if (file_data == NULL) {
                 goto error;
             }
@@ -529,30 +504,30 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
         self->pcur_entry->file_data = file_data;
         self->pcur_entry->file_tracer = file_tracer;
 
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, filename, "traced");
+        SHOWLOG(PyFrame_GetLineNumber(frame), filename, "traced");
     }
     else {
         Py_XDECREF(self->pcur_entry->file_data);
         self->pcur_entry->file_data = NULL;
         self->pcur_entry->file_tracer = Py_None;
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, filename, "skipped");
+        SHOWLOG(PyFrame_GetLineNumber(frame), filename, "skipped");
     }
 
     self->pcur_entry->disposition = disposition;
 
     /* Make the frame right in case settrace(gettrace()) happens. */
     Py_INCREF(self);
-    My_XSETREF(frame->f_trace, (PyObject*)self);
+    Py_XSETREF(frame->f_trace, (PyObject*)self);
 
     /* A call event is really a "start frame" event, and can happen for
      * re-entering a generator also.  f_lasti is -1 for a true call, and a
      * real byte offset for a generator re-entry.
      */
-    if (frame->f_lasti < 0) {
-        self->pcur_entry->last_line = -frame->f_code->co_firstlineno;
+    if (MyFrame_lasti(frame) < 0) {
+        self->pcur_entry->last_line = -MyFrame_GetCode(frame)->co_firstlineno;
     }
     else {
-        self->pcur_entry->last_line = frame->f_lineno;
+        self->pcur_entry->last_line = PyFrame_GetLineNumber(frame);
     }
 
 ok:
@@ -571,52 +546,17 @@ error:
 static void
 CTracer_disable_plugin(CTracer *self, PyObject * disposition)
 {
-    PyObject * file_tracer = NULL;
-    PyObject * plugin = NULL;
-    PyObject * plugin_name = NULL;
-    PyObject * msg = NULL;
-    PyObject * ignored = NULL;
-
+    PyObject * ret;
     PyErr_Print();
 
-    file_tracer = PyObject_GetAttr(disposition, str_file_tracer);
-    if (file_tracer == NULL) {
-        goto error;
-    }
-    if (file_tracer == Py_None) {
-        /* This shouldn't happen... */
-        goto ok;
-    }
-    plugin = PyObject_GetAttr(file_tracer, str__coverage_plugin);
-    if (plugin == NULL) {
-        goto error;
-    }
-    plugin_name = PyObject_GetAttr(plugin, str__coverage_plugin_name);
-    if (plugin_name == NULL) {
-        goto error;
-    }
-    msg = MyText_FromFormat(
-        "Disabling plug-in '%s' due to previous exception",
-        MyText_AsString(plugin_name)
-        );
-    if (msg == NULL) {
-        goto error;
-    }
     STATS( self->stats.pycalls++; )
-    ignored = PyObject_CallFunctionObjArgs(self->warn, msg, NULL);
-    if (ignored == NULL) {
+    ret = PyObject_CallFunctionObjArgs(self->disable_plugin, disposition, NULL);
+    if (ret == NULL) {
         goto error;
     }
+    Py_DECREF(ret);
 
-    /* Disable the plugin for future files, and stop tracing this file. */
-    if (PyObject_SetAttr(plugin, str__coverage_enabled, Py_False) < 0) {
-        goto error;
-    }
-    if (PyObject_SetAttr(disposition, str_trace, Py_False) < 0) {
-        goto error;
-    }
-
-    goto ok;
+    return;
 
 error:
     /* This function doesn't return a status, so if an error happens, print it,
@@ -624,13 +564,6 @@ error:
     /* PySys_WriteStderr is nicer, but is not in the public API. */
     fprintf(stderr, "Error occurred while disabling plug-in:\n");
     PyErr_Print();
-
-ok:
-    Py_XDECREF(file_tracer);
-    Py_XDECREF(plugin);
-    Py_XDECREF(plugin_name);
-    Py_XDECREF(msg);
-    Py_XDECREF(ignored);
 }
 
 
@@ -675,7 +608,7 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
 
     STATS( self->stats.lines++; )
     if (self->pdata_stack->depth >= 0) {
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "line");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "line");
         if (self->pcur_entry->file_data) {
             int lineno_from = -1;
             int lineno_to = -1;
@@ -686,7 +619,8 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
                 STATS( self->stats.pycalls++; )
                 from_to = PyObject_CallMethodObjArgs(self->pcur_entry->file_tracer, str_line_number_range, frame, NULL);
                 if (from_to == NULL) {
-                    goto error;
+                    CTracer_disable_plugin(self, self->pcur_entry->disposition);
+                    goto ok;
                 }
                 ret2 = CTracer_unpack_pair(self, from_to, &lineno_from, &lineno_to);
                 Py_DECREF(from_to);
@@ -696,7 +630,7 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
                 }
             }
             else {
-                lineno_from = lineno_to = frame->f_lineno;
+                lineno_from = lineno_to = PyFrame_GetLineNumber(frame);
             }
 
             if (lineno_from != -1) {
@@ -709,12 +643,12 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
                     }
                     else {
                         /* Tracing lines: key is simply this_line. */
-                        PyObject * this_line = MyInt_FromInt(lineno_from);
+                        PyObject * this_line = PyLong_FromLong((long)lineno_from);
                         if (this_line == NULL) {
                             goto error;
                         }
 
-                        ret2 = PyDict_SetItem(self->pcur_entry->file_data, this_line, Py_None);
+                        ret2 = PySet_Add(self->pcur_entry->file_data, this_line);
                         Py_DECREF(this_line);
                         if (ret2 < 0) {
                             goto error;
@@ -755,14 +689,14 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
              * f_lasti before reading the byte.
              */
             int bytecode = RETURN_VALUE;
-            PyObject * pCode = frame->f_code->co_code;
-            int lasti = frame->f_lasti;
+            PyObject * pCode = MyFrame_GetCode(frame)->co_code;
+            int lasti = MyFrame_lasti(frame);
 
-            if (lasti < MyBytes_GET_SIZE(pCode)) {
-                bytecode = MyBytes_AS_STRING(pCode)[lasti];
+            if (lasti < PyBytes_GET_SIZE(pCode)) {
+                bytecode = PyBytes_AS_STRING(pCode)[lasti];
             }
             if (bytecode != YIELD_VALUE) {
-                int first = frame->f_code->co_firstlineno;
+                int first = MyFrame_GetCode(frame)->co_firstlineno;
                 if (CTracer_record_pair(self, self->pcur_entry->last_line, -first) < 0) {
                     goto error;
                 }
@@ -785,7 +719,7 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
         }
 
         /* Pop the stack. */
-        SHOWLOG(self->pdata_stack->depth, frame->f_lineno, frame->f_code->co_filename, "return");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "return");
         self->pdata_stack->depth--;
         self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
     }
@@ -797,30 +731,6 @@ error:
     return ret;
 }
 
-static int
-CTracer_handle_exception(CTracer *self, PyFrameObject *frame)
-{
-    /* Some code (Python 2.3, and pyexpat anywhere) fires an exception event
-        without a return event.  To detect that, we'll keep a copy of the
-        parent frame for an exception event.  If the next event is in that
-        frame, then we must have returned without a return event.  We can
-        synthesize the missing event then.
-
-        Python itself fixed this problem in 2.4.  Pyexpat still has the bug.
-        I've reported the problem with pyexpat as http://bugs.python.org/issue6359 .
-        If it gets fixed, this code should still work properly.  Maybe some day
-        the bug will be fixed everywhere coverage.py is supported, and we can
-        remove this missing-return detection.
-
-        More about this fix: https://nedbatchelder.com/blog/200907/a_nasty_little_bug.html
-    */
-    STATS( self->stats.exceptions++; )
-    self->last_exc_back = frame->f_back;
-    self->last_exc_firstlineno = frame->f_code->co_firstlineno;
-
-    return RET_OK;
-}
-
 /*
  * The Trace Function
  */
@@ -847,24 +757,21 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
 
     #if WHAT_LOG
     if (what <= (int)(sizeof(what_sym)/sizeof(const char *))) {
-        ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-        printf("trace: %s @ %s %d\n", what_sym[what], MyBytes_AS_STRING(ascii), frame->f_lineno);
+        ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+        printf("%x trace: f:%x %s @ %s %d\n", (int)self, (int)frame, what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
         Py_DECREF(ascii);
     }
     #endif
 
     #if TRACE_LOG
-    ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-    if (strstr(MyBytes_AS_STRING(ascii), start_file) && frame->f_lineno == start_line) {
+    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    if (strstr(PyBytes_AS_STRING(ascii), start_file) && PyFrame_GetLineNumber(frame) == start_line) {
         logging = TRUE;
     }
     Py_DECREF(ascii);
     #endif
 
-    /* See below for details on missing-return detection. */
-    if (CTracer_check_missing_return(self, frame) < 0) {
-        goto error;
-    }
+    self->activity = TRUE;
 
     switch (what) {
     case PyTrace_CALL:
@@ -885,12 +792,6 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
         }
         break;
 
-    case PyTrace_EXCEPTION:
-        if (CTracer_handle_exception(self, frame) < 0) {
-            goto error;
-        }
-        break;
-
     default:
         STATS( self->stats.others++; )
         break;
@@ -952,7 +853,7 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     static char *kwlist[] = {"frame", "event", "arg", "lineno", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O!O|i:Tracer_call", kwlist,
-            &PyFrame_Type, &frame, &MyText_Type, &what_str, &arg, &lineno)) {
+            &PyFrame_Type, &frame, &PyUnicode_Type, &what_str, &arg, &lineno)) {
         goto done;
     }
 
@@ -960,8 +861,8 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
        for the C function. */
     for (what = 0; what_names[what]; what++) {
         int should_break;
-        ascii = MyText_AS_BYTES(what_str);
-        should_break = !strcmp(MyBytes_AS_STRING(ascii), what_names[what]);
+        ascii = PyUnicode_AsASCIIString(what_str);
+        should_break = !strcmp(PyBytes_AS_STRING(ascii), what_names[what]);
         Py_DECREF(ascii);
         if (should_break) {
             break;
@@ -969,8 +870,8 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     }
 
     #if WHAT_LOG
-    ascii = MyText_AS_BYTES(frame->f_code->co_filename);
-    printf("pytrace: %s @ %s %d\n", what_sym[what], MyBytes_AS_STRING(ascii), frame->f_lineno);
+    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    printf("pytrace: %s @ %s %d\n", what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
     Py_DECREF(ascii);
     #endif
 
@@ -1072,10 +973,8 @@ CTracer_get_stats(CTracer *self, PyObject *args_unused)
         "calls", self->stats.calls,
         "lines", self->stats.lines,
         "returns", self->stats.returns,
-        "exceptions", self->stats.exceptions,
         "others", self->stats.others,
         "files", self->stats.files,
-        "missed_returns", self->stats.missed_returns,
         "stack_reallocs", self->stats.stack_reallocs,
         "stack_alloc", self->pdata_stack->alloc,
         "errors", self->stats.errors,
@@ -1119,6 +1018,9 @@ CTracer_members[] = {
     { "switch_context",     T_OBJECT, offsetof(CTracer, switch_context), 0,
             PyDoc_STR("Function for switching to a new context.") },
 
+    { "disable_plugin",     T_OBJECT, offsetof(CTracer, disable_plugin), 0,
+            PyDoc_STR("Function for disabling a plugin.") },
+
     { NULL }
 };
 
@@ -1144,7 +1046,7 @@ CTracer_methods[] = {
 
 PyTypeObject
 CTracerType = {
-    MyType_HEAD_INIT
+    PyVarObject_HEAD_INIT(NULL, 0)
     "coverage.CTracer",        /*tp_name*/
     sizeof(CTracer),           /*tp_basicsize*/
     0,                         /*tp_itemsize*/
