@@ -3,7 +3,10 @@
 
 """Tests for coverage.py's arc measurement."""
 
+import pytest
+
 from tests.coveragetest import CoverageTest
+from tests.helpers import assert_count_equal
 
 import coverage
 from coverage import env
@@ -25,6 +28,7 @@ class SimpleArcTest(CoverageTest):
             b = 3
             """,
             arcz=".1 13 3.")
+        line1 = 1 if env.PYBEHAVIOR.module_firstline_1 else 2
         self.check_coverage("""\
 
             a = 2
@@ -32,7 +36,8 @@ class SimpleArcTest(CoverageTest):
 
             c = 5
             """,
-            arcz="-22 23 35 5-2")
+            arcz="-{0}2 23 35 5-{0}".format(line1)
+        )
 
     def test_function_def(self):
         self.check_coverage("""\
@@ -90,7 +95,7 @@ class SimpleArcTest(CoverageTest):
                 if x % 2: return True
                 return False
             a = fn(1)
-            assert a == True
+            assert a is True
             """,
             arcz=".1 14 45 5.  .2 2. 23 3.", arcz_missing="23 3.")
 
@@ -156,24 +161,65 @@ class SimpleArcTest(CoverageTest):
             arcz_missing=arcz_missing,
         )
 
+    def test_bug_1184(self):
+        self.check_coverage("""\
+            def foo(x):
+                if x:
+                    try:
+                        1/(x - 1)
+                    except ZeroDivisionError:
+                        pass
+                return x        # 7
+
+            for i in range(3):  # 9
+                foo(i)
+            """,
+            arcz=".1 19 9-1  .2 23 27 34 47 56 67 7-1  9A A9",
+            arcz_unpredicted="45",
+            )
+
 
 class WithTest(CoverageTest):
     """Arc-measuring tests involving context managers."""
 
     def test_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 42 2. 16 6."
+        else:
+            arcz = ".1 .2 23 34 4. 16 6."
         self.check_coverage("""\
             def example():
-                with open("test", "w") as f: # exit
-                    f.write("")
-                    return 1
+                with open("test", "w") as f:
+                    f.write("3")
+                    a = 4
 
             example()
             """,
-            arcz=".1 .2 23 34 4. 16 6."
+            arcz=arcz,
+            )
+
+    def test_with_return(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 42 2. 16 6."
+        else:
+            arcz = ".1 .2 23 34 4. 16 6."
+        self.check_coverage("""\
+            def example():
+                with open("test", "w") as f:
+                    f.write("3")
+                    return 4
+
+            example()
+            """,
+            arcz=arcz,
             )
 
     def test_bug_146(self):
-        # https://bitbucket.org/ned/coveragepy/issue/146
+        # https://github.com/nedbat/coveragepy/issues/146
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 32 24 41 15 5."
+        else:
+            arcz = ".1 12 23 34 41 15 5."
         self.check_coverage("""\
             for i in range(2):
                 with open("test", "w") as f:
@@ -181,9 +227,119 @@ class WithTest(CoverageTest):
                 print(4)
             print(5)
             """,
-            arcz=".1 12 23 34 41 15 5."
+            arcz=arcz,
             )
 
+    def test_nested_with_return(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 .2 23 34 45 56 64 42 2. 18 8."
+        else:
+            arcz = ".1 .2 23 34 45 56 6. 18 8."
+        self.check_coverage("""\
+            def example(x):
+                with open("test", "w") as f2:
+                    a = 3
+                    with open("test2", "w") as f4:
+                        f2.write("5")
+                        return 6
+
+            example(8)
+            """,
+            arcz=arcz,
+            )
+
+    def test_break_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 34 42 25 15 5."
+        else:
+            arcz = ".1 12 23 34 45 15 5."
+        self.check_coverage("""\
+            for i in range(1+1):
+                with open("test", "w") as f:
+                    print(3)
+                    break
+            print(5)
+            """,
+            arcz=arcz,
+            arcz_missing="15",
+            )
+
+    def test_continue_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 23 34 42 21 15 5."
+        else:
+            arcz = ".1 12 23 34 41 15 5."
+        self.check_coverage("""\
+            for i in range(1+1):
+                with open("test", "w") as f:
+                    print(3)
+                    continue
+            print(5)
+            """,
+            arcz=arcz,
+            )
+
+    @pytest.mark.skipif(
+        (3, 11) <= env.PYVERSION <= (3, 11, 0, 'alpha', 2, 0),
+        reason="avoid a 3.11 bug: 45709"
+    )
+    # https://github.com/nedbat/coveragepy/issues/1270
+    def test_raise_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 12 27 78 8. 9A A.  -23 34 45 53 6-2"
+            arcz_missing = "6-2 8."
+            arcz_unpredicted = "3-2 89"
+        else:
+            arcz = ".1 12 27 78 8. 9A A.  -23 34 45 5-2 6-2"
+            arcz_missing = "6-2 8."
+            arcz_unpredicted = "89"
+        cov = self.check_coverage("""\
+            from contextlib import suppress
+            def f(x):
+                with suppress():    # used as a null context manager
+                    print(4)
+                    raise Exception("Boo6")
+                print(6)
+            try:
+                f(8)
+            except Exception:
+                print("oops 10")
+            """,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
+            arcz_unpredicted=arcz_unpredicted,
+            )
+        expected = "line 3 didn't jump to the function exit"
+        assert self.get_missing_arc_description(cov, 3, -2) == expected
+
+    def test_untaken_raise_through_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            #arcz = ".1 12 28 89 9. AB B.  -23 3-2 34 45 56 53 63 37 7-2"
+            arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 53 63 37 7-2"
+            #arcz_missing = "3-2 56 63 AB B."
+            arcz_missing = "56 63 AB B."
+        else:
+            arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 6-2 57 7-2"
+            arcz_missing = "56 6-2 AB B."
+        cov = self.check_coverage("""\
+            from contextlib import suppress
+            def f(x):
+                with suppress():    # used as a null context manager
+                    print(4)
+                    if x == 5:
+                        raise Exception("Boo6")
+                print(7)
+            try:
+                f(9)
+            except Exception:
+                print("oops 11")
+            """,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
+            )
+        expected = "line 3 didn't jump to the function exit"
+        assert self.get_missing_arc_description(cov, 3, -2) == expected
+
 
 class LoopArcTest(CoverageTest):
     """Arc-measuring tests involving loops."""
@@ -215,6 +371,13 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_break(self):
+        if env.PYBEHAVIOR.omit_after_jump:
+            arcz = ".1 12 23 35 15 5."
+            arcz_missing = "15"
+        else:
+            arcz = ".1 12 23 35 15 41 5."
+            arcz_missing = "15 41"
+
         self.check_coverage("""\
             for i in range(10):
                 a = i
@@ -222,9 +385,17 @@ class LoopArcTest(CoverageTest):
                 a = 99
             assert a == 0   # 5
             """,
-            arcz=".1 12 23 35 15 41 5.", arcz_missing="15 41")
+            arcz=arcz, arcz_missing=arcz_missing
+        )
 
     def test_continue(self):
+        if env.PYBEHAVIOR.omit_after_jump:
+            arcz = ".1 12 23 31 15 5."
+            arcz_missing = ""
+        else:
+            arcz = ".1 12 23 31 15 41 5."
+            arcz_missing = "41"
+
         self.check_coverage("""\
             for i in range(10):
                 a = i
@@ -232,7 +403,8 @@ class LoopArcTest(CoverageTest):
                 a = 99
             assert a == 9   # 5
             """,
-            arcz=".1 12 23 31 15 41 5.", arcz_missing="41")
+            arcz=arcz, arcz_missing=arcz_missing
+        )
 
     def test_nested_breaks(self):
         self.check_coverage("""\
@@ -246,9 +418,11 @@ class LoopArcTest(CoverageTest):
             """,
             arcz=".1 12 23 34 45 25 56 51 67 17 7.", arcz_missing="17 25")
 
-    def test_while_true(self):
+    def test_while_1(self):
         # With "while 1", the loop knows it's constant.
-        if env.PYBEHAVIOR.nix_while_true:
+        if env.PYBEHAVIOR.keep_constant_test:
+            arcz = ".1 12 23 34 45 36 62 57 7."
+        elif env.PYBEHAVIOR.nix_while_true:
             arcz = ".1 13 34 45 36 63 57 7."
         else:
             arcz = ".1 12 23 34 45 36 63 57 7."
@@ -263,14 +437,16 @@ class LoopArcTest(CoverageTest):
             """,
             arcz=arcz,
             )
+
+    def test_while_true(self):
         # With "while True", 2.x thinks it's computation,
         # 3.x thinks it's constant.
-        if env.PYBEHAVIOR.nix_while_true:
-            arcz = ".1 13 34 45 36 63 57 7."
-        elif env.PY3:
-            arcz = ".1 12 23 34 45 36 63 57 7."
-        else:
+        if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 36 62 57 7."
+        elif env.PYBEHAVIOR.nix_while_true:
+            arcz = ".1 13 34 45 36 63 57 7."
+        else:
+            arcz = ".1 12 23 34 45 36 63 57 7."
         self.check_coverage("""\
             a, i = 1, 0
             while True:
@@ -284,33 +460,36 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_zero_coverage_while_loop(self):
-        # https://bitbucket.org/ned/coveragepy/issue/502
+        # https://github.com/nedbat/coveragepy/issues/502
         self.make_file("main.py", "print('done')")
         self.make_file("zero.py", """\
             def method(self):
                 while True:
                     return 1
             """)
-        out = self.run_command("coverage run --branch --source=. main.py")
-        self.assertEqual(out, 'done\n')
-        if env.PYBEHAVIOR.nix_while_true:
+        cov = coverage.Coverage(source=["."], branch=True)
+        self.start_import_stop(cov, "main")
+        assert self.stdout() == 'done\n'
+        if env.PYBEHAVIOR.keep_constant_test:
+            num_stmts = 3
+        elif env.PYBEHAVIOR.nix_while_true:
             num_stmts = 2
         else:
             num_stmts = 3
         expected = "zero.py {n} {n} 0 0 0% 1-3".format(n=num_stmts)
-        report = self.report_from_command("coverage report -m")
+        report = self.get_report(cov, show_missing=True)
         squeezed = self.squeezed_lines(report)
-        self.assertIn(expected, squeezed[3])
+        assert expected in squeezed[3]
 
     def test_bug_496_continue_in_constant_while(self):
-        # https://bitbucket.org/ned/coveragepy/issue/496
+        # https://github.com/nedbat/coveragepy/issues/496
         # A continue in a while-true needs to jump to the right place.
-        if env.PYBEHAVIOR.nix_while_true:
-            arcz = ".1 13 34 45 53 46 67 7."
-        elif env.PY3:
-            arcz = ".1 12 23 34 45 53 46 67 7."
-        else:
+        if env.PYBEHAVIOR.keep_constant_test:
             arcz = ".1 12 23 34 45 52 46 67 7."
+        elif env.PYBEHAVIOR.nix_while_true:
+            arcz = ".1 13 34 45 53 46 67 7."
+        else:
+            arcz = ".1 12 23 34 45 53 46 67 7."
         self.check_coverage("""\
             up = iter('ta')
             while True:
@@ -344,8 +523,8 @@ class LoopArcTest(CoverageTest):
             branches_3([0,1])
             """,
             arcz=
-                ".1 18 8G GH H. "
-                ".2 23 34 43 26 3. 6. "
+                ".1 18 8G GH H. " +
+                ".2 23 34 43 26 3. 6. " +
                 "-89 9A 9-8 AB BC CB B9 AE E9",
             arcz_missing="26 6."
             )
@@ -382,11 +561,6 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_confusing_for_loop_bug_175(self):
-        if env.PY3:
-            # Py3 counts the list comp as a separate code object.
-            arcz = ".1 -22 2-2 12 23 34 45 53 3."
-        else:
-            arcz = ".1 12 23 34 45 53 3."
         self.check_coverage("""\
             o = [(1,2), (3,4)]
             o = [a for a in o]
@@ -394,19 +568,49 @@ class LoopArcTest(CoverageTest):
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=arcz,
+            arcz=".1 -22 2-2 12 23 34 45 53 3.",
         )
-        if env.PY3:
-            arcz = ".1 12 -22 2-2 23 34 42 2."
-        else:
-            arcz = ".1 12 23 34 42 2."
         self.check_coverage("""\
             o = [(1,2), (3,4)]
             for tup in [a for a in o]:
                 x = tup[0]
                 y = tup[1]
             """,
-            arcz=arcz,
+            arcz=".1 12 -22 2-2 23 34 42 2.",
+        )
+
+    # https://bugs.python.org/issue44672
+    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    def test_incorrect_loop_exit_bug_1175(self):
+        self.check_coverage("""\
+            def wrong_loop(x):
+                if x:
+                    for i in [3, 33]:
+                        print(i+4)
+                else:
+                    pass
+
+            wrong_loop(8)
+            """,
+            arcz=".1 .2 23 26 34 43 3. 6. 18 8.",
+            arcz_missing="26 6.",
+        )
+
+    # https://bugs.python.org/issue44672
+    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    def test_incorrect_if_bug_1175(self):
+        self.check_coverage("""\
+            def wrong_loop(x):
+                if x:
+                    if x:
+                        print(4)
+                else:
+                    pass
+
+            wrong_loop(8)
+            """,
+            arcz=".1 .2 23 26 34 4. 3. 6. 18 8.",
+            arcz_missing="26 3. 6.",
         )
 
     def test_generator_expression(self):
@@ -421,6 +625,21 @@ class LoopArcTest(CoverageTest):
             arcz=".1 -22 2-2 12 23 34 45 53 3.",
         )
 
+    def test_generator_expression_another_way(self):
+        # https://bugs.python.org/issue44450
+        # Generator expression:
+        self.check_coverage("""\
+            o = ((1,2), (3,4))
+            o = (a for
+                 a in
+                 o)
+            for tup in o:
+                x = tup[0]
+                y = tup[1]
+            """,
+            arcz=".1 -22 2-2 12 25 56 67 75 5.",
+        )
+
     def test_other_comprehensions(self):
         # Set comprehension:
         self.check_coverage("""\
@@ -495,6 +714,14 @@ class ExceptionArcTest(CoverageTest):
             assert a == 3 and b == 1
             """,
             arcz=".1 12 23 36 45 56 6.", arcz_missing="45 56")
+
+    def test_raise_followed_by_statement(self):
+        if env.PYBEHAVIOR.omit_after_jump:
+            arcz = ".1 12 23 34 46 67 78 8."
+            arcz_missing = ""
+        else:
+            arcz = ".1 12 23 34 46 58 67 78 8."
+            arcz_missing = "58"
         self.check_coverage("""\
             a, b = 1, 1
             try:
@@ -505,8 +732,7 @@ class ExceptionArcTest(CoverageTest):
                 b = 7
             assert a == 3 and b == 7
             """,
-            arcz=".1 12 23 34 46 58 67 78 8.",
-            arcz_missing="58",
+            arcz=arcz, arcz_missing=arcz_missing,
         )
 
     def test_hidden_raise(self):
@@ -579,15 +805,15 @@ class ExceptionArcTest(CoverageTest):
                 try:
                     a = 4
                     raise Exception("Yikes!")
-                    a = 6
+                    # line 6
                 finally:
                     c = 8
             except:
                 d = 10                              # A
             assert a == 4 and c == 8 and d == 10    # B
             """,
-            arcz=".1 12 23 34 45 58 68 89 8B 9A AB B.",
-            arcz_missing="68 8B",
+            arcz=".1 12 23 34 45 58 89 9A AB B.",
+            arcz_missing="",
         )
 
     def test_finally_in_loop(self):
@@ -653,6 +879,26 @@ class ExceptionArcTest(CoverageTest):
             arcz_missing="3D BC CD",
         )
 
+    def test_break_continue_without_finally(self):
+        self.check_coverage("""\
+            a, c, d, i = 1, 1, 1, 99
+            try:
+                for i in range(3):
+                    try:
+                        a = 5
+                        if i > 0:
+                            break
+                        continue
+                    except:
+                        c = 10
+            except:
+                d = 12                              # C
+            assert a == 5 and c == 1 and d == 1     # D
+            """,
+            arcz=".1 12 23 34 3D 45 56 67 68 7D 83 9A A3 BC CD D.",
+            arcz_missing="3D 9A A3 BC CD",
+        )
+
     def test_continue_through_finally(self):
         if env.PYBEHAVIOR.finally_jumps_back:
             arcz = ".1 12 23 34 3D 45 56 67 68 73 7A 8A A3 A7 BC CD D."
@@ -837,18 +1083,18 @@ class ExceptionArcTest(CoverageTest):
     def test_except_jump_finally(self):
         if env.PYBEHAVIOR.finally_jumps_back:
             arcz = (
-                ".1 1Q QR RS ST TU U. "
-                ".2 23 34 45 56 4O 6L "
-                "78 89 9A AL LA AO  8B BC CD DL LD D4  BE EF FG GL LG G.  EH HI IJ JL  HL "
-                "L4 LM "
+                ".1 1Q QR RS ST TU U. " +
+                ".2 23 34 45 56 4O 6L " +
+                "78 89 9A AL LA AO  8B BC CD DL LD D4  BE EF FG GL LG G.  EH HI IJ JL  HL " +
+                "L4 LM " +
                 "MN NO O."
             )
         else:
             arcz = (
-                ".1 1Q QR RS ST TU U. "
-                ".2 23 34 45 56 4O 6L "
-                "78 89 9A AL  8B BC CD DL  BE EF FG GL  EH HI IJ JL  HL "
-                "LO L4 L. LM "
+                ".1 1Q QR RS ST TU U. " +
+                ".2 23 34 45 56 4O 6L " +
+                "78 89 9A AL  8B BC CD DL  BE EF FG GL  EH HI IJ JL  HL " +
+                "LO L4 L. LM " +
                 "MN NO O."
             )
         self.check_coverage("""\
@@ -891,18 +1137,18 @@ class ExceptionArcTest(CoverageTest):
     def test_else_jump_finally(self):
         if env.PYBEHAVIOR.finally_jumps_back:
             arcz = (
-                ".1 1S ST TU UV VW W. "
-                ".2 23 34 45 56 6A 78 8N 4Q "
-                "AB BC CN NC CQ  AD DE EF FN NF F4  DG GH HI IN NI I.  GJ JK KL LN  JN "
-                "N4 NO "
+                ".1 1S ST TU UV VW W. " +
+                ".2 23 34 45 56 6A 78 8N 4Q " +
+                "AB BC CN NC CQ  AD DE EF FN NF F4  DG GH HI IN NI I.  GJ JK KL LN  JN " +
+                "N4 NO " +
                 "OP PQ Q."
             )
         else:
             arcz = (
-                ".1 1S ST TU UV VW W. "
-                ".2 23 34 45 56 6A 78 8N 4Q "
-                "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN "
-                "N4 NQ N. NO "
+                ".1 1S ST TU UV VW W. " +
+                ".2 23 34 45 56 6A 78 8N 4Q " +
+                "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN " +
+                "N4 NQ N. NO " +
                 "OP PQ Q."
             )
         self.check_coverage("""\
@@ -1040,16 +1286,12 @@ class YieldTest(CoverageTest):
             next(gen)
             print(gen.send(6))
             """,
-            arcz=
-                ".1 17 78 89 9A AB B. "
-                ".2 23 34 45 52 2.",
+            arcz=".1 17 78 89 9A AB B. .2 23 34 45 52 2.",
             arcz_missing="2.",
         )
-        self.assertEqual(self.stdout(), "20\n12\n")
+        assert self.stdout() == "20\n12\n"
 
     def test_yield_from(self):
-        if not env.PYBEHAVIOR.yield_from:
-            self.skipTest("Python before 3.3 doesn't have 'yield from'")
         self.check_coverage("""\
             def gen(inp):
                 i = 2
@@ -1066,7 +1308,7 @@ class YieldTest(CoverageTest):
         )
 
     def test_abandoned_yield(self):
-        # https://bitbucket.org/ned/coveragepy/issue/440
+        # https://github.com/nedbat/coveragepy/issues/440
         self.check_coverage("""\
             def gen():
                 print("yup")
@@ -1082,42 +1324,131 @@ class YieldTest(CoverageTest):
         )
 
 
+@pytest.mark.skipif(env.PYVERSION <= (3, 10, 0, 'beta', 4, 0), reason="3.10.0b4 had bugs")
+@pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
+class MatchCaseTest(CoverageTest):
+    """Tests of match-case."""
+    def test_match_case_with_default(self):
+        self.check_coverage("""\
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                    case _:
+                        match = "default"
+                print(match)
+            """,
+            arcz=".1 12 23 34 49 35 56 69 57 78 89 91 1.",
+        )
+        assert self.stdout() == "default\nno go\ngo: n\n"
+
+    def test_match_case_with_wildcard(self):
+        self.check_coverage("""\
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                    case x:
+                        match = f"default: {x}"
+                print(match)
+            """,
+            arcz=".1 12 23 34 49 35 56 69 57 78 89 91 1.",
+        )
+        assert self.stdout() == "default: ['huh']\nno go\ngo: n\n"
+
+    def test_match_case_without_wildcard(self):
+        self.check_coverage("""\
+            match = None
+            for command in ["huh", "go home", "go n"]:
+                match command.split():
+                    case ["go", direction] if direction in "nesw":
+                        match = f"go: {direction}"
+                    case ["go", _]:
+                        match = "no go"
+                print(match)
+            """,
+            arcz=".1 12 23 34 45 58 46 78 67 68 82 2.",
+        )
+        assert self.stdout() == "None\nno go\ngo: n\n"
+
+
 class OptimizedIfTest(CoverageTest):
     """Tests of if statements being optimized away."""
 
     def test_optimized_away_if_0(self):
+        if env.PYBEHAVIOR.keep_constant_test:
+            lines = [1, 2, 3, 4, 8, 9]
+            arcz = ".1 12 23 24 34 48 49 89 9."
+            arcz_missing = "24"
+            # 49 isn't missing because line 4 is matched by the default partial
+            # exclusion regex, and no branches are considered missing if they
+            # start from an excluded line.
+        else:
+            lines = [1, 2, 3, 8, 9]
+            arcz = ".1 12 23 28 38 89 9."
+            arcz_missing = "28"
+
         self.check_coverage("""\
             a = 1
             if len([2]):
                 c = 3
-            if 0:               # this line isn't in the compiled code.
+            if 0:
                 if len([5]):
                     d = 6
             else:
                 e = 8
             f = 9
             """,
-            lines=[1, 2, 3, 8, 9],
-            arcz=".1 12 23 28 38 89 9.",
-            arcz_missing="28",
+            lines=lines,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
     def test_optimized_away_if_1(self):
+        if env.PYBEHAVIOR.keep_constant_test:
+            lines = [1, 2, 3, 4, 5, 6, 9]
+            arcz = ".1 12 23 24 34 45 49 56 69 59 9."
+            arcz_missing = "24 59"
+            # 49 isn't missing because line 4 is matched by the default partial
+            # exclusion regex, and no branches are considered missing if they
+            # start from an excluded line.
+        else:
+            lines = [1, 2, 3, 5, 6, 9]
+            arcz = ".1 12 23 25 35 56 69 59 9."
+            arcz_missing = "25 59"
+
         self.check_coverage("""\
             a = 1
             if len([2]):
                 c = 3
-            if 1:               # this line isn't in the compiled code,
-                if len([5]):    # but these are.
+            if 1:
+                if len([5]):
                     d = 6
             else:
                 e = 8
             f = 9
             """,
-            lines=[1, 2, 3, 5, 6, 9],
-            arcz=".1 12 23 25 35 56 69 59 9.",
-            arcz_missing="25 59",
+            lines=lines,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
+
+    def test_optimized_away_if_1_no_else(self):
+        if env.PYBEHAVIOR.keep_constant_test:
+            lines = [1, 2, 3, 4, 5]
+            arcz = ".1 12 23 25 34 45 5."
+            arcz_missing = ""
+            # 25 isn't missing because line 2 is matched by the default partial
+            # exclusion regex, and no branches are considered missing if they
+            # start from an excluded line.
+        else:
+            lines = [1, 3, 4, 5]
+            arcz = ".1 13 34 45 5."
+            arcz_missing = ""
         self.check_coverage("""\
             a = 1
             if 1:
@@ -1125,11 +1456,24 @@ class OptimizedIfTest(CoverageTest):
                 c = 4
             d = 5
             """,
-            lines=[1, 3, 4, 5],
-            arcz=".1 13 34 45 5.",
+            lines=lines,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
-    def test_optimized_nested(self):
+    def test_optimized_if_nested(self):
+        if env.PYBEHAVIOR.keep_constant_test:
+            lines = [1, 2, 8, 11, 12, 13, 14, 15]
+            arcz = ".1 12 28 2F 8B 8F BC CD DE EF F."
+            arcz_missing = ""
+            # 2F and 8F aren't missing because they're matched by the default
+            # partial exclusion regex, and no branches are considered missing
+            # if they start from an excluded line.
+        else:
+            lines = [1, 12, 14, 15]
+            arcz = ".1 1C CE EF F."
+            arcz_missing = ""
+
         self.check_coverage("""\
             a = 1
             if 0:
@@ -1147,14 +1491,34 @@ class OptimizedIfTest(CoverageTest):
                     h = 14
             i = 15
             """,
-            lines=[1, 12, 14, 15],
-            arcz=".1 1C CE EF F.",
+            lines=lines,
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
+    def test_dunder_debug(self):
+        # Since some of our tests use __debug__, let's make sure it is true as
+        # we expect
+        assert __debug__
+        # Check that executed code has __debug__
+        self.check_coverage("""\
+            assert __debug__, "assert __debug__"
+            """
+        )
+        # Check that if it didn't have debug, it would let us know.
+        with pytest.raises(AssertionError):
+            self.check_coverage("""\
+                assert not __debug__, "assert not __debug__"
+                """
+            )
+
     def test_if_debug(self):
-        if not env.PYBEHAVIOR.optimize_if_debug:
-            self.skipTest("PyPy doesn't optimize away 'if __debug__:'")
-        # CPython optimizes away "if __debug__:"
+        if env.PYBEHAVIOR.optimize_if_debug:
+            arcz = ".1 12 24 41 26 61 1."
+            arcz_missing = ""
+        else:
+            arcz = ".1 12 23 31 34 41 26 61 1."
+            arcz_missing = "31"
         self.check_coverage("""\
             for value in [True, False]:
                 if value:
@@ -1163,29 +1527,32 @@ class OptimizedIfTest(CoverageTest):
                 else:
                     x = 6
             """,
-            arcz=".1 12 24 41 26 61 1.",
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
     def test_if_not_debug(self):
-        # Before 3.7, no Python optimized away "if not __debug__:"
-        if not env.PYBEHAVIOR.optimize_if_debug:
-            self.skipTest("PyPy doesn't optimize away 'if __debug__:'")
+        arcz_missing = ""
+        if env.PYBEHAVIOR.pep626:
+            arcz = ".1 12 23 34 42 37 72 28 8."
+        elif env.PYBEHAVIOR.optimize_if_not_debug3:
+            arcz = ".1 12 23 32 37 72 28 8."
         elif env.PYBEHAVIOR.optimize_if_not_debug2:
-            arcz = ".1 12 24 41 26 61 1."
-            arcz_missing = ""
+            arcz = ".1 12 23 35 52 37 72 28 8."
         elif env.PYBEHAVIOR.optimize_if_not_debug:
-            arcz = ".1 12 23 31 26 61 1."
-            arcz_missing = ""
+            arcz = ".1 12 23 34 42 37 72 28 8."
         else:
-            arcz = ".1 12 23 31 34 41 26 61 1."
-            arcz_missing = "34 41"
+            arcz = ".1 12 23 34 45 42 52 37 72 28 8."
+            arcz_missing = "45 52"
         self.check_coverage("""\
+            lines = set()
             for value in [True, False]:
                 if value:
                     if not __debug__:
-                        x = 4
+                        lines.add(5)
                 else:
-                    x = 6
+                    lines.add(7)
+            assert lines == set([7])
             """,
             arcz=arcz,
             arcz_missing=arcz_missing,
@@ -1224,8 +1591,6 @@ class MiscArcTest(CoverageTest):
         )
 
     def test_unpacked_literals(self):
-        if not env.PYBEHAVIOR.unpackings_pep448:
-            self.skipTest("Don't have unpacked literals until 3.5")
         self.check_coverage("""\
             d = {
                 'a': 2,
@@ -1255,30 +1620,28 @@ class MiscArcTest(CoverageTest):
             arcz=".1 15 5A A."
         )
 
-    def test_pathologically_long_code_object(self):
-        # https://bitbucket.org/ned/coveragepy/issue/359
-        # The structure of this file is such that an EXTENDED_ARG bytecode is
-        # needed to encode the jump at the end.  We weren't interpreting those
-        # opcodes.
-        # Note that we no longer interpret bytecode at all, but it couldn't
-        # hurt to keep the test...
-        for n in [10, 50, 100, 500, 1000, 2000]:
-            code = """\
-                data = [
-                """ + "".join("""\
-                    [
-                        {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}],
-                """.format(i=i) for i in range(n)
-                ) + """\
-                ]
+    @pytest.mark.parametrize("n", [10, 50, 100, 500, 1000, 2000, 10000])
+    def test_pathologically_long_code_object(self, n):
+        # https://github.com/nedbat/coveragepy/issues/359
+        # Long code objects sometimes cause problems. Originally, it was
+        # due to EXTENDED_ARG bytes codes.  Then it showed a mistake in
+        # line-number packing.
+        code = """\
+            data = [
+            """ + "".join("""\
+                [
+                    {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}, {i}],
+            """.format(i=i) for i in range(n)
+            ) + """\
+            ]
 
-                print(len(data))
-                """
-            self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
-            self.assertEqual(self.stdout().split()[-1], str(n))
+            print(len(data))
+            """
+        self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
+        assert self.stdout() == f"{n}\n"
 
     def test_partial_generators(self):
-        # https://bitbucket.org/ned/coveragepy/issues/475/generator-expression-is-marked-as-not
+        # https://github.com/nedbat/coveragepy/issues/475
         # Line 2 is executed completely.
         # Line 3 is started but not finished, because zip ends before it finishes.
         # Line 4 is never started.
@@ -1294,18 +1657,10 @@ class MiscArcTest(CoverageTest):
             arcz=".1 17 7.  .2 23 34 45 5.  -22 2-2  -33 3-3  -44 4-4",
             arcz_missing="3-3 -44 4-4",
         )
-        # ugh, unexposed methods??
-        filename = self.last_module_name + ".py"
-        fr = cov._get_file_reporter(filename)
-        arcs_executed = cov._analyze(filename).arcs_executed()
-        self.assertEqual(
-            fr.missing_arc_description(3, -3, arcs_executed),
-            "line 3 didn't finish the generator expression on line 3"
-        )
-        self.assertEqual(
-            fr.missing_arc_description(4, -4, arcs_executed),
-            "line 4 didn't run the generator expression on line 4"
-        )
+        expected = "line 3 didn't finish the generator expression on line 3"
+        assert self.get_missing_arc_description(cov, 3, -3) == expected
+        expected = "line 4 didn't run the generator expression on line 4"
+        assert self.get_missing_arc_description(cov, 4, -4) == expected
 
 
 class DecoratorArcTest(CoverageTest):
@@ -1475,20 +1830,23 @@ class LambdaArcTest(CoverageTest):
         )
 
 
+skip_eventlet_670 = pytest.mark.skipif(
+    env.PYVERSION[:2] == (3, 9) and env.OSX,
+    reason="Avoid an eventlet bug on Mac 3.9: eventlet#670",
+    # https://github.com/eventlet/eventlet/issues/670
+)
+
+
 class AsyncTest(CoverageTest):
     """Tests of the new async and await keywords in Python 3.5"""
 
-    def setUp(self):
-        if not env.PYBEHAVIOR.async_syntax:
-            self.skipTest("Async features are new in Python 3.5")
-        super(AsyncTest, self).setUp()
-
+    @skip_eventlet_670
     def test_async(self):
         self.check_coverage("""\
             import asyncio
 
             async def compute(x, y):                        # 3
-                print("Compute %s + %s ..." % (x, y))
+                print(f"Compute {x} + {y} ...")
                 await asyncio.sleep(0.001)
                 return x + y                                # 6
 
@@ -1496,20 +1854,21 @@ class AsyncTest(CoverageTest):
                 result = (0 +
                             await compute(x, y)             # A
                 )
-                print("%s + %s = %s" % (x, y, result))
+                print(f"{x} + {y} = {result}")
 
             loop = asyncio.new_event_loop()                 # E
             loop.run_until_complete(print_sum(1, 2))
             loop.close()                                    # G
             """,
             arcz=
-                ".1 13 38 8E EF FG G. "
-                "-34 45 56 6-3 "
+                ".1 13 38 8E EF FG G. " +
+                "-34 45 56 6-3 " +
                 "-89 9C C-8",
             arcz_unpredicted="5-3 9-8",
         )
-        self.assertEqual(self.stdout(), "Compute 1 + 2 ...\n1 + 2 = 3\n")
+        assert self.stdout() == "Compute 1 + 2 ...\n1 + 2 = 3\n"
 
+    @skip_eventlet_670
     def test_async_for(self):
         self.check_coverage("""\
             import asyncio
@@ -1545,16 +1904,22 @@ class AsyncTest(CoverageTest):
                 "-AB BC C-A DE E-A ",       # __anext__
             arcz_unpredicted="CD",
         )
-        self.assertEqual(self.stdout(), "a\nb\nc\n.\n")
+        assert self.stdout() == "a\nb\nc\n.\n"
 
     def test_async_with(self):
+        if env.PYBEHAVIOR.exit_through_with:
+            arcz = ".1 1. .2 23 32 2."
+            arcz_missing = ".2 23 32 2."
+        else:
+            arcz = ".1 1. .2 23 3."
+            arcz_missing = ".2 23 3."
         self.check_coverage("""\
             async def go():
                 async with x:
                     pass
             """,
-            arcz=".1 1. .2 23 3.",
-            arcz_missing=".2 23 3.",
+            arcz=arcz,
+            arcz_missing=arcz_missing,
         )
 
     def test_async_decorator(self):
@@ -1574,6 +1939,98 @@ class AsyncTest(CoverageTest):
             arcz_missing='-46 6-4',
         )
 
+    # https://github.com/nedbat/coveragepy/issues/1158
+    # https://bugs.python.org/issue44621
+    @pytest.mark.skipif(env.PYVERSION[:2] == (3, 9), reason="avoid a 3.9 bug: 44621")
+    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
+    def test_bug1158(self):
+        self.check_coverage("""\
+            import asyncio
+
+            async def async_gen():
+                yield 4
+
+            async def async_test():
+                global a
+                a = 8
+                async for i in async_gen():
+                    print(i + 10)
+                else:
+                    a = 12
+
+            asyncio.run(async_test())
+            assert a == 12
+            """,
+            arcz=".1 13 36 6E EF F.  -34 4-3  -68 89 9A 9C A9 C-6",
+        )
+        assert self.stdout() == "14\n"
+
+    # https://github.com/nedbat/coveragepy/issues/1176
+    # https://bugs.python.org/issue44622
+    @pytest.mark.skipif(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "beta", 4, 0),
+        reason="avoid a 3.10 bug fixed after beta 4: 44622"
+    )
+    @skip_eventlet_670
+    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
+    def test_bug1176(self):
+        self.check_coverage("""\
+            import asyncio
+
+            async def async_gen():
+                yield 4
+
+            async def async_test():
+                async for i in async_gen():
+                    print(i + 8)
+
+            asyncio.run(async_test())
+            """,
+            arcz=".1 13 36 6A A.  -34 4-3  -67 78 87 7-6",
+        )
+        assert self.stdout() == "12\n"
+
+    # https://github.com/nedbat/coveragepy/issues/1205
+    # https://bugs.python.org/issue44840
+    @pytest.mark.skipif(
+        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "candidate", 1, 0),
+        reason="avoid a 3.10 bug fixed after rc1: 44840"
+    )
+    def test_bug1205(self):
+        self.check_coverage("""\
+            def func():
+                if T(2):
+                    if T(3):
+                        if F(4):
+                            if X(5):
+                                return 6
+                    else:
+                        return 8
+                elif X(9) and Y:
+                    return 10
+
+            T, F = (lambda _: True), (lambda _: False)
+            func()
+            """,
+            arcz=".1 1C CD D.  .2 23 29 34 38 45 4. 56 5. 6. 8. 9. 9A A.  -CC C-C",
+            arcz_missing="29 38 45 56 5. 6. 8. 9. 9A A.",
+        )
+
+
+class AnnotationTest(CoverageTest):
+    """Tests using type annotations."""
+
+    def test_annotations(self):
+        self.check_coverage("""\
+            def f(x:str, y:int) -> str:
+                a:int = 2
+                return f"{x}, {y}, {a}, 3"
+            print(f("x", 4))
+            """,
+            arcz=".1 .2 23 3.  14 4.",
+        )
+        assert self.stdout() == "x, 4, 2, 3\n"
+
 
 class ExcludeTest(CoverageTest):
     """Tests of exclusions to indicate known partial branches."""
@@ -1627,4 +2084,4 @@ class LineDataTest(CoverageTest):
 
         data = cov.get_data()
         fun1_lines = data.lines(abs_file("fun1.py"))
-        self.assertCountEqual(fun1_lines, [1, 2, 5])
+        assert_count_equal(fun1_lines, [1, 2, 5])
