@@ -6,17 +6,20 @@
 import glob
 import os
 import random
+import re
 import sys
 import threading
 import time
 
 from flaky import flaky
+import pytest
 
 import coverage
 from coverage import env
-from coverage.backward import import_local_file
 from coverage.data import line_counts
+from coverage.exceptions import ConfigError
 from coverage.files import abs_file
+from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
 from tests.helpers import remove_files
@@ -91,7 +94,7 @@ class LineCountTest(CoverageTest):
             print("done")
             """
 
-        self.assertEqual(line_count(CODE), 5)
+        assert line_count(CODE) == 5
 
 
 # The code common to all the concurrency models.
@@ -144,13 +147,7 @@ PRINT_SUM_RANGE = """
     """
 
 # Import the things to use threads.
-if env.PY2:
-    THREAD = """
-    import threading
-    import Queue as queue
-    """
-else:
-    THREAD = """
+THREAD = """
     import threading
     import queue
     """
@@ -191,15 +188,13 @@ def cant_trace_msg(concurrency, the_module):
         # We don't even have the underlying module installed, we expect
         # coverage to alert us to this fact.
         expected_out = (
-            "Couldn't trace with concurrency=%s, "
-            "the module isn't installed.\n" % concurrency
+            f"Couldn't trace with concurrency={concurrency}, the module isn't installed.\n"
         )
     elif env.C_TRACER or concurrency == "thread" or concurrency == "":
         expected_out = None
     else:
         expected_out = (
-            "Can't support concurrency=%s with PyTracer, "
-            "only threads are supported\n" % concurrency
+            f"Can't support concurrency={concurrency} with PyTracer, only threads are supported.\n"
         )
     return expected_out
 
@@ -218,23 +213,23 @@ class ConcurrencyTest(CoverageTest):
         is the text we expect the code to produce.
 
         """
-
         self.make_file("try_it.py", code)
 
-        cmd = "coverage run --concurrency=%s try_it.py" % concurrency
+        cmd = f"coverage run --concurrency={concurrency} try_it.py"
         out = self.run_command(cmd)
 
         expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
         if expected_cant_trace is not None:
-            self.assertEqual(out, expected_cant_trace)
+            assert out == expected_cant_trace
+            pytest.skip(f"Can't test: {expected_cant_trace}")
         else:
             # We can fully measure the code if we are using the C tracer, which
             # can support all the concurrency, or if we are using threads.
             if expected_out is None:
                 expected_out = "%d\n" % (sum(range(self.QLIMIT)))
             print(code)
-            self.assertEqual(out, expected_out)
+            assert out == expected_out
 
             # Read the coverage file and see that try_it.py has all its lines
             # executed.
@@ -244,11 +239,11 @@ class ConcurrencyTest(CoverageTest):
             # If the test fails, it's helpful to see this info:
             fname = abs_file("try_it.py")
             linenos = data.lines(fname)
-            print("{}: {}".format(len(linenos), linenos))
+            print(f"{len(linenos)}: {linenos}")
             print_simple_annotation(code, linenos)
 
             lines = line_count(code)
-            self.assertEqual(line_counts(data)['try_it.py'], lines)
+            assert line_counts(data)['try_it.py'] == lines
 
     def test_threads(self):
         code = (THREAD + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
@@ -266,6 +261,8 @@ class ConcurrencyTest(CoverageTest):
         code = SIMPLE.format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "eventlet", eventlet)
 
+    # https://github.com/nedbat/coveragepy/issues/663
+    @pytest.mark.skipif(env.WINDOWS, reason="gevent has problems on Windows: #663")
     def test_gevent(self):
         code = (GEVENT + SUM_RANGE_Q + PRINT_SUM_RANGE).format(QLIMIT=self.QLIMIT)
         self.try_some_code(code, "gevent", gevent)
@@ -314,6 +311,85 @@ class ConcurrencyTest(CoverageTest):
             """
         self.try_some_code(BUG_330, "eventlet", eventlet, "0\n")
 
+    def test_threads_with_gevent(self):
+        self.make_file("both.py", """\
+            import queue
+            import threading
+
+            import gevent
+
+            def work1(q):
+                q.put(1)
+
+            def gwork(q):
+                gevent.spawn(work1, q).join()
+                q.put(None)
+                print("done")
+
+            q = queue.Queue()
+            t = threading.Thread(target=gwork, args=(q,))
+            t.start()
+            t.join()
+
+            answer = q.get()
+            assert answer == 1
+            """)
+        out = self.run_command("coverage run --concurrency=thread,gevent both.py")
+        if gevent is None:
+            assert out == (
+                "Couldn't trace with concurrency=gevent, the module isn't installed.\n"
+            )
+            pytest.skip("Can't run test without gevent installed.")
+        if not env.C_TRACER:
+            assert out == (
+                "Can't support concurrency=gevent with PyTracer, only threads are supported.\n"
+            )
+            pytest.skip("Can't run gevent with PyTracer")
+
+        assert out == "done\n"
+
+        out = self.run_command("coverage report -m")
+        last_line = self.squeezed_lines(out)[-1]
+        assert re.search(r"TOTAL \d+ 0 100%", last_line)
+
+    def test_bad_concurrency(self):
+        self.make_file("prog.py", "a = 1")
+        msg = "Unknown concurrency choices: nothing"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run --concurrency=nothing prog.py")
+
+    def test_bad_concurrency_in_config(self):
+        self.make_file("prog.py", "a = 1")
+        self.make_file(".coveragerc", "[run]\nconcurrency = nothing\n")
+        msg = "Unknown concurrency choices: nothing"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run prog.py")
+
+    def test_no_multiple_light_concurrency(self):
+        self.make_file("prog.py", "a = 1")
+        msg = "Conflicting concurrency settings: eventlet, gevent"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run --concurrency=gevent,eventlet prog.py")
+
+    def test_no_multiple_light_concurrency_in_config(self):
+        self.make_file("prog.py", "a = 1")
+        self.make_file(".coveragerc", "[run]\nconcurrency = gevent, eventlet\n")
+        msg = "Conflicting concurrency settings: eventlet, gevent"
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line("run prog.py")
+
+
+class WithoutConcurrencyModuleTest(CoverageTest):
+    """Tests of what happens if the requested concurrency isn't installed."""
+
+    @pytest.mark.parametrize("module", ["eventlet", "gevent", "greenlet"])
+    def test_missing_module(self, module):
+        self.make_file("prog.py", "a = 1")
+        sys.modules[module] = None
+        msg = f"Couldn't trace with concurrency={module}, the module isn't installed."
+        with pytest.raises(ConfigError, match=msg):
+            self.command_line(f"run --concurrency={module} prog.py")
+
 
 SQUARE_OR_CUBE_WORK = """
     def work(x):
@@ -340,8 +416,8 @@ MULTI_CODE = """
     def process_worker_main(args):
         # Need to pause, or the tasks go too quickly, and some processes
         # in the pool don't get any work, and then don't record data.
-        time.sleep(0.02)
         ret = work(*args)
+        time.sleep(0.1)
         return os.getpid(), ret
 
     if __name__ == "__main__":      # pragma: no branch
@@ -362,32 +438,23 @@ MULTI_CODE = """
     """
 
 
+@pytest.mark.skipif(not multiprocessing, reason="No multiprocessing in this Python")
 @flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
 class MultiprocessingTest(CoverageTest):
     """Test support of the multiprocessing module."""
 
-    def setUp(self):
-        if not multiprocessing:
-            self.skipTest("No multiprocessing in this Python")      # pragma: only jython
-        super(MultiprocessingTest, self).setUp()
-
     def try_multiprocessing_code(
         self, code, expected_out, the_module, nprocs, concurrency="multiprocessing", args=""
     ):
         """Run code using multiprocessing, it should produce `expected_out`."""
         self.make_file("multi.py", code)
-        self.make_file(".coveragerc", """\
+        self.make_file(".coveragerc", f"""\
             [run]
-            concurrency = %s
+            concurrency = {concurrency}
             source = .
-            """ % concurrency)
+            """)
 
-        if env.PYVERSION >= (3, 4):
-            start_methods = ['fork', 'spawn']
-        else:
-            start_methods = ['']
-
-        for start_method in start_methods:
+        for start_method in ["fork", "spawn"]:
             if start_method and start_method not in multiprocessing.get_all_start_methods():
                 continue
 
@@ -399,24 +466,31 @@ class MultiprocessingTest(CoverageTest):
             expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
             if expected_cant_trace is not None:
-                self.assertEqual(out, expected_cant_trace)
+                print(out)
+                assert out == expected_cant_trace
+                pytest.skip(f"Can't test: {expected_cant_trace}")
             else:
-                self.assertEqual(out.rstrip(), expected_out)
-                self.assertEqual(len(glob.glob(".coverage.*")), nprocs + 1)
+                assert out.rstrip() == expected_out
+                assert len(glob.glob(".coverage.*")) == nprocs + 1
 
                 out = self.run_command("coverage combine")
-                self.assertEqual(out, "")
+                out_lines = out.splitlines()
+                assert len(out_lines) == nprocs + 1
+                assert all(
+                    re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                    for line in out_lines
+                )
                 out = self.run_command("coverage report -m")
 
                 last_line = self.squeezed_lines(out)[-1]
-                self.assertRegex(last_line, r"multi.py \d+ 0 100%")
+                assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
     def test_multiprocessing_simple(self):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(code, expected_out, threading, nprocs)
 
     def test_multiprocessing_append(self):
@@ -424,7 +498,7 @@ class MultiprocessingTest(CoverageTest):
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(code, expected_out, threading, nprocs, args="--append")
 
     def test_multiprocessing_and_gevent(self):
@@ -434,13 +508,21 @@ class MultiprocessingTest(CoverageTest):
             SUM_RANGE_WORK + EVENTLET + SUM_RANGE_Q + MULTI_CODE
         ).format(NPROCS=nprocs, UPTO=upto)
         total = sum(sum(range((x + 1) * 100)) for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
+        expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(
             code, expected_out, eventlet, nprocs, concurrency="multiprocessing,eventlet"
         )
 
-    def try_multiprocessing_code_with_branching(self, code, expected_out):
-        """Run code using multiprocessing, it should produce `expected_out`."""
+    @pytest.mark.parametrize("start_method", ["fork", "spawn"])
+    def test_multiprocessing_with_branching(self, start_method):
+        if start_method not in multiprocessing.get_all_start_methods():
+            pytest.skip(f"{start_method} not supported here")
+
+        nprocs = 3
+        upto = 30
+        code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
+        total = sum(x*x if x%2 else x*x*x for x in range(upto))
+        expected_out = f"{nprocs} pids, total = {total}"
         self.make_file("multi.py", code)
         self.make_file("multi.rc", """\
             [run]
@@ -449,32 +531,15 @@ class MultiprocessingTest(CoverageTest):
             omit = */site-packages/*
             """)
 
-        if env.PYVERSION >= (3, 4):
-            start_methods = ['fork', 'spawn']
-        else:
-            start_methods = ['']
+        out = self.run_command(f"coverage run --rcfile=multi.rc multi.py {start_method}")
+        assert out.rstrip() == expected_out
 
-        for start_method in start_methods:
-            if start_method and start_method not in multiprocessing.get_all_start_methods():
-                continue
+        out = self.run_command("coverage combine -q")   # sneak in a test of -q
+        assert out == ""
+        out = self.run_command("coverage report -m")
 
-            out = self.run_command("coverage run --rcfile=multi.rc multi.py %s" % (start_method,))
-            self.assertEqual(out.rstrip(), expected_out)
-
-            out = self.run_command("coverage combine")
-            self.assertEqual(out, "")
-            out = self.run_command("coverage report -m")
-
-            last_line = self.squeezed_lines(out)[-1]
-            self.assertRegex(last_line, r"multi.py \d+ 0 \d+ 0 100%")
-
-    def test_multiprocessing_with_branching(self):
-        nprocs = 3
-        upto = 30
-        code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
-        total = sum(x*x if x%2 else x*x*x for x in range(upto))
-        expected_out = "{nprocs} pids, total = {total}".format(nprocs=nprocs, total=total)
-        self.try_multiprocessing_code_with_branching(code, expected_out)
+        last_line = self.squeezed_lines(out)[-1]
+        assert re.search(r"TOTAL \d+ 0 \d+ 0 100%", last_line)
 
     def test_multiprocessing_bootstrap_error_handling(self):
         # An exception during bootstrapping will be reported.
@@ -490,8 +555,8 @@ class MultiprocessingTest(CoverageTest):
             _crash = _bootstrap
             """)
         out = self.run_command("coverage run multi.py")
-        self.assertIn("Exception during multiprocessing bootstrap init", out)
-        self.assertIn("Exception: Crashing because called by _bootstrap", out)
+        assert "Exception during multiprocessing bootstrap init" in out
+        assert "Exception: Crashing because called by _bootstrap" in out
 
     def test_bug890(self):
         # chdir in multiprocessing shouldn't keep us from finding the
@@ -510,7 +575,7 @@ class MultiprocessingTest(CoverageTest):
             concurrency = multiprocessing
             """)
         out = self.run_command("coverage run multi.py")
-        self.assertEqual(out.splitlines()[-1], "ok")
+        assert out.splitlines()[-1] == "ok"
 
 
 def test_coverage_stop_in_threads():
@@ -520,7 +585,7 @@ def test_coverage_stop_in_threads():
     def run_thread():           # pragma: nested
         """Check that coverage is stopping properly in threads."""
         deadline = time.time() + 5
-        ident = threading.currentThread().ident
+        ident = threading.current_thread().ident
         if sys.gettrace() is not None:
             has_started_coverage.append(ident)
         while sys.gettrace() is not None:
@@ -545,12 +610,11 @@ def test_coverage_stop_in_threads():
 
 
 def test_thread_safe_save_data(tmpdir):
-    # Non-regression test for:
-    # https://bitbucket.org/ned/coveragepy/issues/581
+    # Non-regression test for: https://github.com/nedbat/coveragepy/issues/581
 
     # Create some Python modules and put them in the path
     modules_dir = tmpdir.mkdir('test_modules')
-    module_names = ["m{:03d}".format(i) for i in range(1000)]
+    module_names = [f"m{i:03d}" for i in range(1000)]
     for module_name in module_names:
         modules_dir.join(module_name + ".py").write("def f(): pass\n")
 
