@@ -5,7 +5,7 @@
 
 import pytest
 
-from coverage.misc import CoverageException
+from coverage.exceptions import ConfigError
 from coverage.results import format_lines, Numbers, should_fail_under
 
 from tests.coveragetest import CoverageTest
@@ -18,65 +18,70 @@ class NumbersTest(CoverageTest):
 
     def test_basic(self):
         n1 = Numbers(n_files=1, n_statements=200, n_missing=20)
-        self.assertEqual(n1.n_statements, 200)
-        self.assertEqual(n1.n_executed, 180)
-        self.assertEqual(n1.n_missing, 20)
-        self.assertEqual(n1.pc_covered, 90)
+        assert n1.n_statements == 200
+        assert n1.n_executed == 180
+        assert n1.n_missing == 20
+        assert n1.pc_covered == 90
 
     def test_addition(self):
         n1 = Numbers(n_files=1, n_statements=200, n_missing=20)
         n2 = Numbers(n_files=1, n_statements=10, n_missing=8)
         n3 = n1 + n2
-        self.assertEqual(n3.n_files, 2)
-        self.assertEqual(n3.n_statements, 210)
-        self.assertEqual(n3.n_executed, 182)
-        self.assertEqual(n3.n_missing, 28)
-        self.assertAlmostEqual(n3.pc_covered, 86.666666666)
+        assert n3.n_files == 2
+        assert n3.n_statements == 210
+        assert n3.n_executed == 182
+        assert n3.n_missing == 28
+        assert round(abs(n3.pc_covered-86.666666666), 7) == 0
 
     def test_sum(self):
         n1 = Numbers(n_files=1, n_statements=200, n_missing=20)
         n2 = Numbers(n_files=1, n_statements=10, n_missing=8)
         n3 = sum([n1, n2])
-        self.assertEqual(n3.n_files, 2)
-        self.assertEqual(n3.n_statements, 210)
-        self.assertEqual(n3.n_executed, 182)
-        self.assertEqual(n3.n_missing, 28)
-        self.assertAlmostEqual(n3.pc_covered, 86.666666666)
+        assert n3.n_files == 2
+        assert n3.n_statements == 210
+        assert n3.n_executed == 182
+        assert n3.n_missing == 28
+        assert round(abs(n3.pc_covered-86.666666666), 7) == 0
 
-    def test_pc_covered_str(self):
-        # Numbers._precision is a global, which is bad.
-        Numbers.set_precision(0)
-        n0 = Numbers(n_files=1, n_statements=1000, n_missing=0)
-        n1 = Numbers(n_files=1, n_statements=1000, n_missing=1)
-        n999 = Numbers(n_files=1, n_statements=1000, n_missing=999)
-        n1000 = Numbers(n_files=1, n_statements=1000, n_missing=1000)
-        self.assertEqual(n0.pc_covered_str, "100")
-        self.assertEqual(n1.pc_covered_str, "99")
-        self.assertEqual(n999.pc_covered_str, "1")
-        self.assertEqual(n1000.pc_covered_str, "0")
+    @pytest.mark.parametrize("kwargs, res", [
+        (dict(n_files=1, n_statements=1000, n_missing=0), "100"),
+        (dict(n_files=1, n_statements=1000, n_missing=1), "99"),
+        (dict(n_files=1, n_statements=1000, n_missing=999), "1"),
+        (dict(n_files=1, n_statements=1000, n_missing=1000), "0"),
+        (dict(precision=1, n_files=1, n_statements=10000, n_missing=0), "100.0"),
+        (dict(precision=1, n_files=1, n_statements=10000, n_missing=1), "99.9"),
+        (dict(precision=1, n_files=1, n_statements=10000, n_missing=9999), "0.1"),
+        (dict(precision=1, n_files=1, n_statements=10000, n_missing=10000), "0.0"),
+    ])
+    def test_pc_covered_str(self, kwargs, res):
+        assert Numbers(**kwargs).pc_covered_str == res
 
-    def test_pc_covered_str_precision(self):
-        # Numbers._precision is a global, which is bad.
-        Numbers.set_precision(1)
-        n0 = Numbers(n_files=1, n_statements=10000, n_missing=0)
-        n1 = Numbers(n_files=1, n_statements=10000, n_missing=1)
-        n9999 = Numbers(n_files=1, n_statements=10000, n_missing=9999)
-        n10000 = Numbers(n_files=1, n_statements=10000, n_missing=10000)
-        self.assertEqual(n0.pc_covered_str, "100.0")
-        self.assertEqual(n1.pc_covered_str, "99.9")
-        self.assertEqual(n9999.pc_covered_str, "0.1")
-        self.assertEqual(n10000.pc_covered_str, "0.0")
-        Numbers.set_precision(0)
+    @pytest.mark.parametrize("prec, pc, res", [
+        (0, 47.87, "48"),
+        (1, 47.87, "47.9"),
+        (0, 99.995, "99"),
+        (2, 99.99995, "99.99"),
+    ])
+    def test_display_covered(self, prec, pc, res):
+        assert Numbers(precision=prec).display_covered(pc) == res
+
+    @pytest.mark.parametrize("prec, width", [
+        (0, 3),     # 100
+        (1, 5),     # 100.0
+        (4, 8),     # 100.0000
+    ])
+    def test_pc_str_width(self, prec, width):
+        assert Numbers(precision=prec).pc_str_width() == width
 
     def test_covered_ratio(self):
         n = Numbers(n_files=1, n_statements=200, n_missing=47)
-        self.assertEqual(n.ratio_covered, (153, 200))
+        assert n.ratio_covered == (153, 200)
 
         n = Numbers(
             n_files=1, n_statements=200, n_missing=47,
             n_branches=10, n_missing_branches=3, n_partial_branches=1000,
         )
-        self.assertEqual(n.ratio_covered, (160, 210))
+        assert n.ratio_covered == (160, 210)
 
 
 @pytest.mark.parametrize("total, fail_under, precision, result", [
@@ -109,12 +114,12 @@ def test_should_fail_under(total, fail_under, precision, result):
 
 
 def test_should_fail_under_invalid_value():
-    with pytest.raises(CoverageException, match=r"fail_under=101"):
+    with pytest.raises(ConfigError, match=r"fail_under=101"):
         should_fail_under(100.0, 101, 0)
 
 
 @pytest.mark.parametrize("statements, lines, result", [
-    (set([1,2,3,4,5,10,11,12,13,14]), set([1,2,5,10,11,13,14]), "1-2, 5-11, 13-14"),
+    ({1,2,3,4,5,10,11,12,13,14}, {1,2,5,10,11,13,14}, "1-2, 5-11, 13-14"),
     ([1,2,3,4,5,10,11,12,13,14,98,99], [1,2,5,10,11,13,14,99], "1-2, 5-11, 13-14, 99"),
     ([1,2,3,4,98,99,100,101,102,103,104], [1,2,99,102,103,104], "1-2, 99, 102-104"),
     ([17], [17], "17"),
@@ -128,8 +133,8 @@ def test_format_lines(statements, lines, result):
 
 @pytest.mark.parametrize("statements, lines, arcs, result", [
     (
-        set([1,2,3,4,5,10,11,12,13,14]),
-        set([1,2,5,10,11,13,14]),
+        {1,2,3,4,5,10,11,12,13,14},
+        {1,2,5,10,11,13,14},
         (),
         "1-2, 5-11, 13-14"
         ),
